<?php

// Model to handle interactions with MyMix profiles
class MyMix_Profile {

	private $default_xml = <<<XML
<?xml version="1.0" encoding="iso-8859-1"?>
<profile>
    <ch1name>CHANNEL 1</ch1name>
    <ch2name>CHANNEL 2</ch2name>
    <inputmode>dual</inputmode>
    <phantompower>off</phantompower>
    <effecttype>None</effecttype>
    <effectdelaytime>10</effectdelaytime>
    <effectdelayrepeat>0</effectdelayrepeat>
    <playalong>off</playalong>
    <transmitstereomix>off</transmitstereomix>
    <parametriceq>
        <bypass>false</bypass>
        <lowmastereqtype>Peaking</lowmastereqtype>
        <highmastereqtype>Peaking</highmastereqtype>
        <band index="0">
            <frequencyindex>16</frequencyindex>
            <gainindex>36</gainindex>
            <qindex>10</qindex>
        </band>
        <band index="1">
            <frequencyindex>16</frequencyindex>
            <gainindex>36</gainindex>
            <qindex>10</qindex>
        </band>
        <band index="2">
            <frequencyindex>16</frequencyindex>
            <gainindex>36</gainindex>
            <qindex>10</qindex>
        </band>
        <band index="3">
            <frequencyindex>16</frequencyindex>
            <gainindex>36</gainindex>
            <qindex>10</qindex>
        </band>
    </parametriceq>
    <remotechannels>
        <channel>
            <name>CHANNEL 1</name>
            <unitname>DEFAULT</unitname>
            <selected>0</selected>
            <volume>10</volume>
            <pan>50</pan>
            <tone>50</tone>
            <effectsend>0</effectsend>
            <mute>off</mute>
            <solo>off</solo>
            <inputmode>dual</inputmode>
            <mac>ff:ff:ff:ff:ff:ff</mac>
        </channel>
	</remotechannels>
</profile>
XML;

	private $MAX_REMOTE_CHANNELS = 128;
	private $MAX_CHAN_NAME_LEN = 16;
	private $INPUT_MODE_VALUES = array("dual", "stereo", "mono", "none");
	private $PHANTOM_POWER_VALUES = array("on", "off");
	private $EFFECT_TYPE_VALUES = array("None", "Small Room", "Medium Room", "Large Room", "Small Hall", "Medium Hall", "Large Hall", "Delay", );
	private $EFFECT_DELAY_TIME_MIN = 10;
	private $EFFECT_DELAY_TIME_MAX = 350;
	private $EFFECT_DELAY_REPEAT_MIN = 0;
	private $EFFECT_DELAY_REPEAT_MAX = 100;
	private $PLAY_ALONG_VALUES = array("on", "off");
	private $TRANSMIT_STEREO_MIX_VALUES = array("on", "off");
	private $BYPASS_VALUES = array("true", "false");
	private $EQ_TYPE_VALUES = array("Peaking", "Shelving");
	private $NUM_BANDS = 4;
	private $BAND_INDEX_MIN = 0;
	private $BAND_INDEX_MAX = 3;
	private $FREQUENCY_INDEX_MIN = 0;
	private $FREQUENCY_INDEX_MAX = 387;
	private $GAIN_INDEX_MIN = 0;
	private $GAIN_INDEX_MAX = 48;
	private $Q_INDEX_MIN = 0;
	private $Q_INDEX_MAX = 196;

	private $MAX_UNIT_NAME_LEN = 16;
	private $SELECTED_VALUES = array(0, 1);
	private $VOLUME_MIN = 0;
	private $VOLUME_MAX = 100;
	private $PAN_MIN = 0;
	private $PAN_MAX = 100;
	private $EFFECT_SEND_MIN = 0;
	private $EFFECT_SEND_MAX = 100;
	private $MUTE_VALUES = array("on", "off");
	private $SOLO_VALUES = array("on", "off");

	private $xml_data;
	private $xml_path;

	function __construct($filename)
	{
		if(file_exists($filename))
		{
			$this->load($filename);
		}
		else
		{
			$this->xml_data = new SimpleXMLElement($this->default_xml, LIBXML_NOEMPTYTAG);
			$this->xml_path = $filename;
		}
	}

	// Returns a SimpleXMLElement that refers to the base <profile> element
	// and contains a structure like that of default_xml, above.
	private function load($filename)
	{
		libxml_use_internal_errors(true);
		$retval = simplexml_load_file($filename, "SimpleXMLElement", LIBXML_NOEMPTYTAG);
		if($retval === FALSE)
		{
			$errors = libxml_get_errors();
			$error_message = "";
			foreach ($errors as $error)
			{
				$error_message .= $error->message . " in " . $error->file . " at line " . $error->line . ".\n";
			}

			throw new Exception("Failed to open file: " . $error_message);
		}

		$this->xml_data = $retval;
		$this->xml_path = $filename;
	}

	// Writes profile out to filename.
	public function save()
	{
		require_once("helpers/xml.php");

		$path = dirname($this->xml_path);
		if(!is_dir($path))
		{
			mkdir($path, 0777, TRUE);
		}

		$dom = dom_import_simplexml($this->xml_data);
		$temp_output = $dom->ownerDocument->saveXML($dom->ownerDocument, LIBXML_NOEMPTYTAG);
		$temp_output = XML::pretty_format($temp_output);
		$result = file_put_contents($this->xml_path, $temp_output);
		if($result === FALSE)
		{
			$error = error_get_last();
			throw new Exception("Failed to open file: " . $error['message']);
		}
	}

	public function set_ch1_name($name)
	{
		if(strlen($name) > $this->MAX_CHAN_NAME_LEN)
		{
			$name = substr($name, 0, $this->MAX_CHAN_NAME_LEN);
		}

		$this->xml_data->ch1name = $name;
	}

	public function get_ch1_name()
	{
		return (string)$this->xml_data->ch1name;
	}

	public function set_ch2_name($name)
	{
		if(strlen($name) > $this->MAX_CHAN_NAME_LEN)
		{
			$name = substr($name, 0, $this->MAX_CHAN_NAME_LEN);
		}

		$this->xml_data->ch2name = $name;
	}

	public function get_ch2_name()
	{
		return (string)$this->xml_data->ch2name;
	}

	public function set_input_mode($mode)
	{
		if(!in_array($mode, $this->INPUT_MODE_VALUES))
		{
			return;
		}
		$this->xml_data->inputmode = $mode;
	}

	public function get_input_mode()
	{
		return (string)$this->xml_data->inputmode;
	}

	public function set_phantom_power($power)
	{
		if(!in_array($power, $this->PHANTOM_POWER_VALUES))
		{
			die("Invalid phantom power value!");
		}
		$this->xml_data->phantompower = $power;
	}

	public function get_phantom_power()
	{
		return (string)$this->xml_data->phantompower;
	}

	public function set_effect_type($type)
	{
		if(!in_array($type, $this->EFFECT_TYPE_VALUES))
		{
			die("Invalid effect type value!");
		}
		$this->xml_data->effecttype = $type;
	}

	public function get_effect_type()
	{
		return (string)$this->xml_data->effecttype;
	}

	public function set_effect_delay_time($time)
	{
		if(!is_numeric($time) || $time < $this->EFFECT_DELAY_TIME_MIN || $time > $this->EFFECT_DELAY_TIME_MAX)
		{
			die("Invalid effect delay time value!");
		}
		$this->xml_data->effectdelaytime = $time;
	}

	public function get_effect_delay_time()
	{
		return (int)$this->xml_data->effectdelaytime;
	}

	public function set_effect_delay_repeat($repeat)
	{
		if(!is_numeric($repeat) || $repeat < $this->EFFECT_DELAY_REPEAT_MIN || $repeat > $this->EFFECT_DELAY_REPEAT_MAX)
		{
			die("Invalid effect delay repeat value!");
		}
		$this->xml_data->effectdelayrepeat = $repeat;
	}

	public function get_effect_delay_repeat()
	{
		return (int)$this->xml_data->effectdelayrepeat;
	}

	public function set_play_along($play_along)
	{
		if(!in_array($play_along, $this->PLAY_ALONG_VALUES))
		{
			die("Invalid play along value!");
		}

		$this->xml_data->playalong = $play_along;
	}

	public function get_play_along()
	{
		return (string)$this->xml_data->playalong;
	}

	public function set_transmit_stereo_mix($value)
	{
		if(!in_array($value, $this->TRANSMIT_STEREO_MIX_VALUES))
		{
			die("Invalid transmit stereo mix value!");
		}
		$this->xml_data->transmitstereomix = $value;
	}

	public function get_transmit_stereo_mix()
	{
		return (string)$this->xml_data->transmitstereomix;
	}

	public function set_eq_bypass($value)
	{
		if(!in_array($value, $this->BYPASS_VALUES))
		{
			die("Invalid eq bypass value!");
		}
		$this->xml_data->parametriceq->bypass = $value;
	}

	public function get_eq_bypass()
	{
		return (string)$this->xml_data->parametriceq->bypass;
	}

	public function set_low_master_eq_type($value)
	{
		if(!in_array($value, $this->EQ_TYPE_VALUES))
		{
			die("Invalid low master eq type value!");
		}
		$this->xml_data->parametriceq->lowmastereqtype = $value;
	}

	public function get_low_master_eq_type()
	{
		return (string)$this->xml_data->parametriceq->lowmastereqtype;
	}

	public function set_high_master_eq_type($value)
	{
		if(!in_array($value, $this->EQ_TYPE_VALUES))
		{
			die("Invalid high master eq type value!");
		}
		$this->xml_data->parametriceq->highmastereqtype = $value;
	}

	public function get_high_master_eq_type()
	{
		return (string)$this->xml_data->parametriceq->highmastereqtype;
	}

	public function set_band_index($band, $value)
	{
		if(!is_numeric($value) || $value < 0 || $value >= $this->NUM_BANDS)
		{
			die("Invalid band index value!");
		}
		$this->xml_data->parametriceq->band[$band]['index'] = $value;
	}

	public function get_band_index($band)
	{
		return (int)$this->xml_data->parametriceq->band[$band]['index'];
	}

	public function set_band_frequency_index($band, $value)
	{
		if(!is_numeric($value) || $value < $this->FREQUENCY_INDEX_MIN || $value > $this->FREQUENCY_INDEX_MAX)
		{
			die("Invalid band frequency index value!");
		}
		$this->xml_data->parametriceq->band[$band]->frequencyindex = $value;
	}

	public function get_band_frequency_index($band)
	{
		return (int)$this->xml_data->parametriceq->band[$band]->frequencyindex;
	}

	public function set_band_gain_index($band, $value)
	{
		if(!is_numeric($value) || $value < $this->GAIN_INDEX_MIN || $value > $this->GAIN_INDEX_MAX)
		{
			die("Invalid band gain index value!");
		}

		$this->xml_data->parametriceq->band[$band]->gainindex = $value;
	}

	public function get_band_gain_index($band)
	{
		return (int)$this->xml_data->parametriceq->band[$band]->gainindex;
	}

	public function set_band_q_index($band, $value)
	{
		if(!is_numeric($value) || $value < $this->Q_INDEX_MIN || $value > $this->Q_INDEX_MAX)
		{
			die("Invalid band gain index value!");
		}
		$this->xml_data->parametriceq->band[$band]->qindex = $value;
	}

	public function get_band_q_index($band)
	{
		return (int)$this->xml_data->parametriceq->band[$band]->qindex;
	}

	public function get_remote_channel_count()
	{
		return count($this->xml_data->remotechannels->children());
	}

	public function set_remote_channel_name($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->name = $value;
	}

	public function get_remote_channel_name($channel)
	{
		return (string)$this->xml_data->remotechannels->channel[$channel]->name;
	}

	public function set_remote_channel_unit_name($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->unitname = $value;
	}

	public function get_remote_channel_unit_name($channel)
	{
		return (string)$this->xml_data->remotechannels->channel[$channel]->unitname;
	}

	public function set_remote_channel_selected($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->selected = $value;
	}

	public function get_remote_channel_selected($channel)
	{
		return (int)$this->xml_data->remotechannels->channel[$channel]->selected;
	}

	public function set_remote_channel_volume($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->volume = $value;
	}

	public function get_remote_channel_volume($channel)
	{
		return (int)$this->xml_data->remotechannels->channel[$channel]->volume;
	}

	public function set_remote_channel_pan($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->pan = $value;
	}

	public function get_remote_channel_pan($channel)
	{
		return (int)$this->xml_data->remotechannels->channel[$channel]->pan;
	}

	public function set_remote_channel_tone($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->tone = $value;
	}

	public function get_remote_channel_tone($channel)
	{
		return (int)$this->xml_data->remotechannels->channel[$channel]->tone;
	}

	public function set_remote_channel_effect_send($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->effectsend = $value;
	}

	public function get_remote_channel_effect_send($channel)
	{
		return (int)$this->xml_data->remotechannels->channel[$channel]->effectsend;
	}

	public function set_remote_channel_mute($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->mute = $value;
	}

	public function get_remote_channel_mute($channel)
	{
		return (string)$this->xml_data->remotechannels->channel[$channel]->mute;
	}

	public function set_remote_channel_solo($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->solo = $value;
	}

	public function get_remote_channel_solo($channel)
	{
		return (string)$this->xml_data->remotechannels->channel[$channel]->solo;
	}

	public function set_remote_channel_input_mode($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->inputmode = $value;
	}

	public function get_remote_channel_input_mode($channel)
	{
		return (string)$this->xml_data->remotechannels->channel[$channel]->inputmode;
	}

	public function set_remote_channel_mac($channel, $value)
	{
		$this->xml_data->remotechannels->channel[$channel]->mac = $value;
	}

	public function get_remote_channel_mac($channel)
	{
		return (string)$this->xml_data->remotechannels->channel[$channel]->mac;
	}

	public function add_remote_channel($unit_name, $channel_name, $mac, $selected, $input_mode)
	{
		if($this->get_remote_channel_count() >= $this->MAX_REMOTE_CHANNELS)
		{
			return;
		}

		$channel = $this->xml_data->remotechannels->addChild("channel");
		$channel->addChild("name", $channel_name);
		$channel->addChild("unitname", $unit_name);
		$channel->addChild("selected", $selected);
		$channel->addChild("volume", 10);
		$channel->addChild("pan", 50);
		$channel->addChild("tone", 50);
		$channel->addChild("effectsend", 0);
		$channel->addChild("mute", "off");
		$channel->addChild("solo", "off");
		$channel->addChild("inputmode", $input_mode);
		$channel->addChild("mac", $mac);
	}

	// Helper function for other profile list functions
	private static function profile_list_helper($folder)
	{
		$retval = array();

		$profiles = array();
		if(is_dir($folder))
		{
			exec("ls " . $folder, $profiles);
		}

		foreach($profiles as $this_profile)
		{
			$retval[$this_profile] = $this_profile;
		}

		return $retval;
	}

	public static function get_profile_list($mac)
	{
		require_once("helpers/config.php");

		$path = Config::get_item('mymix_devices_dir') . $mac . "/profiles/";
		return self::profile_list_helper($path);
	}

	public static function get_local_saved_profiles()
	{
		require_once("helpers/config.php");

		$path = Config::get_item('saved_mymix_profiles_dir');
		return self::profile_list_helper($path);
	}

	public static function get_external_saved_profiles()
	{
		require_once("helpers/config.php");

		$path = Config::get_item('external_mymix_profiles_dir');
		return self::profile_list_helper($path);
	}

	public static function get_global_profile_list()
	{
		$retval = array();

		$devices = array();
		$path = Config::get_item('mymix_devices_dir');
		exec("ls " . $path, $devices);
		foreach($devices as $mac)
		{
			$profiles = self::get_profile_list($mac);
			foreach($profiles as $this_profile)
			{
				if(!in_array($this_profile, $retval))
				{
					$retval[$this_profile] = $this_profile;
				}
			}
		}

		return $retval;
	}

	public static function get_frequency_options($band)
	{
		$retval = array();

		$start_index = 0;
		$end_index = 387;
		if($band == 0)
		{
			$end_index = 107;
		}
		elseif($band == 3)
		{
			$start_index = 106;
		}

		for($i = $start_index; $i < $end_index; $i++)
		{
			if($i >= 0 && $i < 16)
			{
				$hz = $i * 5 + 20; // up to 100
			}
			elseif($i >= 16 && $i < 106)
			{
				$hz = ($i - 16) * 10 + 100; // 100 up to 1000
			}
			elseif($i >= 106 && $i < 286)
			{
				$hz = ($i - 106) * 50 + 1000; // 1000 up to 10000
			}
			elseif($i >= 286)
			{
				$hz = ($i - 286) * 100 + 10000; // 10000 to 20000
			}

			if($hz >= 1000)
			{
				$hz = number_format($hz / 1000, 3) . "k";
			}
			$hz .= "Hz";

			$retval[$i - $start_index] = $hz;
		}

		return $retval;
	}

	public static function get_q_options()
	{
		$retval = array();

		for($i = 0; $i <= 196; $i++)
		{
			$retval[$i] = number_format(($i * 0.1 + 0.4), 1);
		}

		return $retval;
	}

	public static function get_gain_options()
	{
		$retval = array();

		for($i = 0; $i <= 48; $i++)
		{
			$retval[$i] = sprintf('%+.1fdB', ($i * 0.5 - 18.0));
		}

		return $retval;
	}

	// Delete a profile from a device in the active project
	public static function delete_profile($mac, $profile_name)
	{
		require_once("helpers/config.php");
		require_once("helpers/validate.php");
		
		if(!Validate::mac_address($mac) || empty($profile_name))
		{
			throw new Exception("Tried to delete profile without MAC or profile name!");
		}

		$path = Config::get_item('mymix_devices_dir') . $mac . "/profiles/" . $profile_name;
		unlink($path);
	}

	// Rename a profile on a device in the active project
	public static function rename_profile($mac, $profile_name, $new_profile_name)
	{
		require_once("helpers/config.php");
		require_once("helpers/validate.php");
		if(!Validate::mac_address($mac) || empty($profile_name) || empty($new_profile_name))
		{
			throw new Exception("Tried to rename profile without MAC, profile name or new profile name!");
		}
		$source_path = Config::get_item('mymix_devices_dir') . $mac . "/profiles/" . $profile_name;
		$dest_path = Config::get_item('mymix_devices_dir') . $mac . "/profiles/" . $new_profile_name;
		rename($source_path, $dest_path);
	}

	public static function delete_local_saved_profile($profile_name)
	{
		require_once("helpers/config.php");
		if(empty($profile_name))
		{
			throw new Exception("Tried to delete local saved profile without profile name!");
		}
		$path = Config::get_item('saved_mymix_profiles_dir') . $profile_name;
		unlink($path);
	}
}

/* End of mymix_profile.php*/
